/*
 * Identify transparent pages using the "usespagetransparency" and
 * "usesanytransparency" pseudo objects.
 *
 * Required software: pCOS interface 8 (TET 4.1, TET PDF IFilter 4.1,
 * PDFlib+PDI 9.0, PPS 9.0)
 *
 * Required data: PDF document
 */
package com.pdflib.cookbook.pcos.pages;

import com.pdflib.IpCOS;
import com.pdflib.cookbook.pcos.pcos_cookbook_example;

public class page_transparency extends pcos_cookbook_example {

    /* This is where the data files are. Adjust as necessary. */
    private final static String SEARCH_PATH = "../input";

    public void example_code(IpCOS p, int doc) throws Exception {

        System.out.println("File name: " + p.pcos_get_string(doc, "filename"));

        int pagecount = (int) p.pcos_get_number(doc, "length:pages");

        for (int page = 0; page < pagecount; page++) {
            System.out.println("Page " + (page + 1) + ": ");

            boolean usespagetransparency =
                p.pcos_get_number(doc, "pages[" + page + "]/usespagetransparency") != 0;
            boolean usesanytransparency =
                p.pcos_get_number(doc, "pages[" + page + "]/usesanytransparency") != 0;
            
            System.out.println("  Transparency in page contents: "
                + (usespagetransparency ? "true" : "false"));

            System.out.println("  Transparency in page contents or annotations: "
                + (usesanytransparency ? "true" : "false"));
            
            System.out.println();
        }
    }

    public page_transparency(String[] argv, String readable_name, String search_path) {
        super(argv, readable_name, search_path);
    }

    public static void main(String argv[]) {
        page_transparency example = new page_transparency(argv, "Page transparency", SEARCH_PATH);
        example.execute();
    }
}
